% File src/library/base/man/chartr.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2009 R Core Team
% Distributed under GPL 2 or later

\name{chartr}
\alias{chartr}
\alias{tolower}
\alias{toupper}
\alias{casefold}
\title{Character Translation and Casefolding}
\description{
  文字列を文字列ベクトルに翻訳する．特に大文字を小文字に，小文字を大文字に変える．
}
\usage{
chartr(old, new, x)
tolower(x)
toupper(x)
casefold(x, upper = FALSE)
}
\arguments{
  \item{x}{文字列ベクトル，もしくは \code{\link{as.character}} で
    文字列に変換できるオブジェクト．}
  \item{old}{翻訳されるべき文字を指定する文字列．
    もし長さ2以上の文字列ベクトルが与えられると，最初の要素が使われ警告が出る．}
  \item{new}{翻訳を指定する文字列．
    もし長さ2以上の文字列ベクトルが与えられると，最初の要素が使われ警告が出る．}
  \item{upper}{論理値: 大文字に変換するか小文字に変換するか?}
}
\details{
  \code{chartr} は \code{old} で指定された \code{x} 中の各文字を
  \code{new} で指定された対応する文字に変換する．
  範囲は指示の中で \code{new} ポートされるが，文字クラス繰り返し文字はそうではない．
  もし \code{old} が \code{new} よりも多くの文字を持てばエラーシグナルが
  発生する； 逆にもし少なければ  \code{new} の後端の余分の文字は無視される．

  \code{tolower} (\code{toupper}) は文字列ベクトル中の大(子)文字を
  小(大)文字に変える．アルファベットでない文字はそのままである．

  \code{casefold} は S-PLUS との互換性のため提供されている
  \code{tolower} と \code{toupper} のためのラッパである．
}
\value{
  (可能な強制変換の) \code{x} と同じ長さと属性の文字列ベクトル．

  もし対応する入力が宣言されたエンコーディングを持ちそして現在の
  ロケールが Latin1 か UTF-8 ならば，結果の要素は現在のロケールで
  宣言されたエンコーディングを持つ(\code{\link{Encoding}} を見よ)．
  対応する入力が UTF-8 で無い限り，結果は現在のロケールのエンコーディング
  を持つ．システムが Unicode の幅広文字を持ちエンコーディングが UTF-8 なら
  結果のエンコーディングは UTF-8 である．
}
\seealso{
  他の文字列中の挿入については \code{\link{sub}} と \code{\link{gsub}}．
}
\examples{
x <- "MiXeD cAsE 123"
chartr("iXs", "why", x)
chartr("a-cX", "D-Fw", x)
tolower(x)
toupper(x)

## "大小文字が混在する" 大文字化 - toupper(単語のすべての最初の文字) :

.simpleCap <- function(x) {
    s <- strsplit(x, " ")[[1]]
    paste(toupper(substring(s, 1, 1)), substring(s, 2),
          sep = "", collapse = " ")
}
.simpleCap("the quick red fox jumps over the lazy brown dog")
## ->  [1] "The Quick Red Fox Jumps Over The Lazy Brown Dog"

## そしてより良く賢明なバージョン:
capwords <- function(s, strict = FALSE) {
    cap <- function(s) paste(toupper(substring(s, 1, 1)),
                  {s <- substring(s, 2); if(strict) tolower(s) else s},
                             sep = "", collapse = " " )
    sapply(strsplit(s, split = " "), cap, USE.NAMES = !is.null(names(s)))
}
capwords(c("using AIC for model selection"))
## ->  [1] "Using AIC For Model Selection"
capwords(c("using AIC", "for MODEL selection"), strict = TRUE)
## ->  [1] "Using Aic"  "For Model Selection"
##                ^^^        ^^^^^
##               'bad'       'good'

## -- 非常に簡単な安全でない暗号化 --
rot <- function(ch, k = 13) {
   p0 <- function(...) paste(c(...), collapse = "")
   A <- c(letters, LETTERS, " '")
   I <- seq_len(k); chartr(p0(A), p0(c(A[-I], A[I])), ch)
}

pw <- "my secret pass phrase"
(crypw <- rot(pw, 13)) #-> これを送ることが出来る

## そして ``解読'' :
rot(crypw, 54 - 13) # -> オリジナル
stopifnot(identical(pw, rot(crypw, 54 - 13)))
}
\keyword{character}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/chartr.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2009 R Core Team
%% Distributed under GPL 2 or later
%
%\name{chartr}
%\alias{chartr}
%\alias{tolower}
%\alias{toupper}
%\alias{casefold}
%\title{Character Translation and Casefolding}
%\description{
%  Translate characters in character vectors, in particular from upper to
%  lower case or vice versa.
%}
%\usage{
%chartr(old, new, x)
%tolower(x)
%toupper(x)
%casefold(x, upper = FALSE)
%}
%\arguments{
%  \item{x}{a character vector, or an object that can be coerced to
%    character by \code{\link{as.character}}.}
%  \item{old}{a character string specifying the characters to be
%    translated.  If a character vector of length 2 or more is supplied,
%    the first element is used with a warning.}
%  \item{new}{a character string specifying the translations. If a
%    character vector of length 2 or more is supplied, the first element
%    is used with a warning.}
%  \item{upper}{logical: translate to upper or lower case?.}
%}
%\details{
%  \code{chartr} translates each character in \code{x} that is specified
%  in \code{old} to the corresponding character specified in \code{new}.
%  Ranges are supported in the specifications, but character classes and
%  repeated characters are not.  If \code{old} contains more characters
%  than new, an error is signaled; if it contains fewer characters, the
%  extra characters at the end of \code{new} are ignored.
%
%  \code{tolower} and \code{toupper} convert upper-case characters in a
%  character vector to lower-case, or vice versa.  Non-alphabetic
%  characters are left unchanged.
%
%  \code{casefold} is a wrapper for \code{tolower} and \code{toupper}
%  provided for compatibility with S-PLUS.
%}
%\value{
%  A character vector of the same length and with the same attributes as
%  \code{x} (after possible coercion).
%
%  Elements of the result will be have the encoding declared as that of
%  the current locale (see \code{\link{Encoding}} if the corresponding
%  input had a declared encoding and the current locale is either Latin-1
%  or UTF-8.  The result will be in the current locale's encoding unless
%  the corresponding input was in UTF-8, when it will be in UTF-8 when
%  the system has Unicode wide characters.
%}
%\seealso{
%  \code{\link{sub}} and \code{\link{gsub}} for other
%  substitutions in strings.
%}
%\examples{
%x <- "MiXeD cAsE 123"
%chartr("iXs", "why", x)
%chartr("a-cX", "D-Fw", x)
%tolower(x)
%toupper(x)
%
%## "Mixed Case" Capitalizing - toupper( every first letter of a word ) :
%
%.simpleCap <- function(x) {
%    s <- strsplit(x, " ")[[1]]
%    paste(toupper(substring(s, 1, 1)), substring(s, 2),
%          sep = "", collapse = " ")
%}
%.simpleCap("the quick red fox jumps over the lazy brown dog")
%## ->  [1] "The Quick Red Fox Jumps Over The Lazy Brown Dog"
%
%## and the better, more sophisticated version:
%capwords <- function(s, strict = FALSE) {
%    cap <- function(s) paste(toupper(substring(s, 1, 1)),
%                  {s <- substring(s, 2); if(strict) tolower(s) else s},
%                             sep = "", collapse = " " )
%    sapply(strsplit(s, split = " "), cap, USE.NAMES = !is.null(names(s)))
%}
%capwords(c("using AIC for model selection"))
%## ->  [1] "Using AIC For Model Selection"
%capwords(c("using AIC", "for MODEL selection"), strict = TRUE)
%## ->  [1] "Using Aic"  "For Model Selection"
%##                ^^^        ^^^^^
%##               'bad'       'good'
%
%## -- Very simple insecure crypto --
%rot <- function(ch, k = 13) {
%   p0 <- function(...) paste(c(...), collapse = "")
%   A <- c(letters, LETTERS, " '")
%   I <- seq_len(k); chartr(p0(A), p0(c(A[-I], A[I])), ch)
%}
%
%pw <- "my secret pass phrase"
%(crypw <- rot(pw, 13)) #-> you can send this off
%
%## now ``decrypt'' :
%rot(crypw, 54 - 13) # -> the original:
%stopifnot(identical(pw, rot(crypw, 54 - 13)))
%}
%\keyword{character}
%
\encoding{UTF8}
