% File src/library/base/man/eval.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2011 R Core Team
% Distributed under GPL 2 or later

\name{eval}
\alias{eval}
\alias{evalq}
\alias{eval.parent}
\alias{local}
\title{Evaluate an (Unevaluated) Expression}
\description{
  \R 表現式を指定された環境中で評価する．
}
\usage{
eval(expr, envir = parent.frame(),
           enclos = if(is.list(envir) || is.pairlist(envir))
                       parent.frame() else baseenv())
evalq(expr, envir, enclos)
eval.parent(expr, n = 1)
local(expr, envir = new.env())
}
\arguments{
  \item{expr}{評価されるオブジェクト．\sQuote{詳細}を見よ．}
  \item{envir}{\code{expr} がその中で評価される環境．
    \code{NULL}, リスト, データフレーム，対リスト，
    \code{\link{sys.call}} で指定されるような整数，でも良い．}
  \item{enclos}{\code{envir} が(対)リストやデータフレームのとき関係する．
    囲み環境を指定する，つまり \R が \code{envir} 中で見つけられなかったオブジェクトを
    探す場所．これは \code{NULL} (基本パッケージ環境 \code{\link{baseenv}()} と解釈される) や環境でも良い．}
  \item{n}{遡るべき親世代の数．}
}
\details{
  \code{eval} は \code{expr} 引数を \code{envir} で指定された慣用で評価し，
  計算された値を返す．もし \code{envir} が指定されないと，既定は \code{\link{parent.frame}()}
  (\code{eval} の呼び出しが行われた環境である)．

  評価されるべきオブジェクトは型 \code{\link{call}} か 
  \code{\link{expression}} 又は \link{name} (名前は現在のスコープで探され
  それに結び付けられたものが評価される)，\link{promise} 又は
  ベクトル，関数又は環境(そのまま返される)．

  形式 \code{evalq} は \code{eval(quote(expr), \dots)} と同値である．
  \code{eval}はその最初の引数を評価器に渡す前に現在のスコープで評価する：
  \code{evalq} はこれを避ける．

  \code{eval.parent(expr, n)} は
  \code{eval(expr, parent.frame(n))} の短縮形である．

  もし \code{envir} がリスト(データフレームのような)又は対リストならば，それは
  一時的環境にコピーされ(囲み環境 \code{enclos} で)，
  一時的環境が評価に用いられる．従って
  \code{expr} は(対)リスト中の名前を持つ任意の成分を変更し，
  変更は失われる．

  もし \code{envir} が \code{NULL} ならばそれは空のリストと解釈され，
  従って \code{envir} 中には何の値も存在せず探索は直接囲み環境に
  向かう．

  \code{local} は表現式を局所的環境で評価する．これは
  その既定引数が新しい空の環境を作ることを除けば \code{evalq} に等しい，
  これは匿名の再帰的関数を作ったり，環境中で定義された変数は外部からは見えないため
  一種の限定された名前空間の特性を作るのに役立つ．
}
\value{
  オブジェクトの評価結果：表現式ベクトルに対してはこれは
  最後の要素の評価結果．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.  (\code{eval} only.)
}
\seealso{
  \code{\link{expression}}, \code{\link{quote}}, \code{\link{sys.frame}},
  \code{\link{parent.frame}}, \code{\link{environment}}.

  更に，典型的には関数引数の，評価を強制するには \code{\link{force}}．
}
\note{
  スコープ規則の差から，この領域では \R と S の間には
  幾つかの違いがある．特に，S での既定囲み環境は大局的環境である．

  関数引数として渡されているデータフレーム中で表現式を評価する際は，
  関連する囲み環境はしばしば呼び出し側の環境である．つまり，
  \code{eval(x, data, parent.frame())} が必要になる．
}
\examples{
eval(2 ^ 2 ^ 3)
mEx <- expression(2^2^3); mEx; 1 + eval(mEx)
eval({ xx <- pi; xx^2}) ; xx

a <- 3 ; aa <- 4 ; evalq(evalq(a+b+aa, list(a = 1)), list(b = 5)) # == 10
a <- 3 ; aa <- 4 ; evalq(evalq(a+b+aa, -1), list(b = 5))        # == 12

ev <- function() {
   e1 <- parent.frame()
   ## Evaluate a in e1
   aa <- eval(expression(a), e1)
   ## evaluate the expression bound to a in e1
   a <- expression(x+y)
   list(aa = aa, eval = eval(a, e1))
}
tst.ev <- function(a = 7) { x <- pi; y <- 1; ev() }
tst.ev()  #-> aa : 7,  eval : 4.14

a <- list(a = 3, b = 4)
with(a, a <- 5) # a のコピーをリストから作り，捨て去る．

##
## evalq() の例
##

N <- 3
env <- new.env()
assign("N", 27, envir = env)
## このバージョンは N の可視的なコピーだけを変更する，
## なぜなら eval に渡される引数は '4' だから．.
eval(N <- 4, env)
N
get("N", envir = env)
## このバージョンは env に付値する，そしてそこで N だけを変更する．
evalq(N <- 5, env)
N
get("N", envir = env)


##
## local() の用法
##

# 相互再帰的．
# gg は最後の付値の値を受け取る，f の匿名バージョン．

gg <- local({
    k <- function(y)f(y)
    f <- function(x) if(x) x*k(x-1) else 1
})
gg(10)
sapply(1:5, gg)

# 入れ子の local：a は k にアクセスできるプライベートな保管庫
gg <- local({
    k <- local({
        a <- 1
        function(y){print(a <<- a+1);f(y)}
    })
    f <- function(x) if(x) x*k(x-1) else 1
})
sapply(1:5, gg)

ls(envir = environment(gg))
ls(envir = environment(get("k", envir = environment(gg))))
}
\keyword{data}
\keyword{programming}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/eval.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2011 R Core Team
%% Distributed under GPL 2 or later
%
%\name{eval}
%\alias{eval}
%\alias{evalq}
%\alias{eval.parent}
%\alias{local}
%\title{Evaluate an (Unevaluated) Expression}
%\description{
%  Evaluate an \R expression in a specified environment.
%}
%\usage{
%eval(expr, envir = parent.frame(),
%           enclos = if(is.list(envir) || is.pairlist(envir))
%                       parent.frame() else baseenv())
%evalq(expr, envir, enclos)
%eval.parent(expr, n = 1)
%local(expr, envir = new.env())
%}
%\arguments{
%  \item{expr}{an object to be evaluated.  See \sQuote{Details}.}
%  \item{envir}{the \code{\link{environment}} in which \code{expr} is to
%    be evaluated.  May also be \code{NULL}, a list, a data frame,
%    a pairlist or an integer as specified to \code{\link{sys.call}}.}
%  \item{enclos}{Relevant when \code{envir} is a (pair)list or a data frame.
%    Specifies the enclosure, i.e., where \R looks for objects not found
%    in \code{envir}.  This can be \code{NULL} (interpreted as the base
%    package environment, \code{\link{baseenv}()}) or an environment.}
%  \item{n}{number of parent generations to go back}
%}
%\details{
%  \code{eval} evaluates the \code{expr} argument in the
%  environment specified by \code{envir} and returns the computed value.
%  If \code{envir} is not specified, then the default is
%  \code{\link{parent.frame}()} (the environment where the call to
%  \code{eval} was made).
%
%  Objects to be evaluated can be of types \code{\link{call}} or
%  \code{\link{expression}} or \link{name} (when the name is looked
%  up in the current scope and its binding is evaluated), a \link{promise}
%  or any of the basic types such as vectors, functions and environments
%  (which are returned unchanged).
%
%  The \code{evalq} form is equivalent to \code{eval(quote(expr), \dots)}.
%  \code{eval} evaluates its first argument in the current scope
%  before passing it to the evaluator: \code{evalq} avoids this.
%
%  \code{eval.parent(expr, n)} is a shorthand for
%  \code{eval(expr, parent.frame(n))}.
%
%  If \code{envir} is a list (such as a data frame) or pairlist, it is
%  copied into a temporary environment (with enclosure \code{enclos}),
%  and the temporary environment is used for evaluation.  So if
%  \code{expr} changes any of the components named in the (pair)list, the
%  changes are lost.
%
%  If \code{envir} is \code{NULL} it is interpreted as an empty list so
%  no values could be found in \code{envir} and look-up goes directly to
%  \code{enclos}.
%
%  \code{local} evaluates an expression in a local environment.  It is
%  equivalent to \code{evalq} except that its default argument creates a
%  new, empty environment.  This is useful to create anonymous recursive
%  functions and as a kind of limited namespace feature since variables
%  defined in the environment are not visible from the outside.
%}
%\value{
%  The result of evaluating the object: for an expression vector this is
%  the result of evaluating the last element.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.  (\code{eval} only.)
%}
%\seealso{
%  \code{\link{expression}}, \code{\link{quote}}, \code{\link{sys.frame}},
%  \code{\link{parent.frame}}, \code{\link{environment}}.
%
%  Further, \code{\link{force}} to \emph{force} evaluation, typically of
%  function arguments.
%}
%\note{
%  Due to the difference in scoping rules, there are some differences
%  between \R and S in this area.  In particular, the default enclosure
%  in S is the global environment.
%
%  When evaluating expressions in a data frame that has been passed as an
%  argument to a function, the relevant enclosure is often the caller's
%  environment, i.e., one needs
%  \code{eval(x, data, parent.frame())}.
%}
%\examples{
%eval(2 ^ 2 ^ 3)
%mEx <- expression(2^2^3); mEx; 1 + eval(mEx)
%eval({ xx <- pi; xx^2}) ; xx
%
%a <- 3 ; aa <- 4 ; evalq(evalq(a+b+aa, list(a = 1)), list(b = 5)) # == 10
%a <- 3 ; aa <- 4 ; evalq(evalq(a+b+aa, -1), list(b = 5))        # == 12
%
%ev <- function() {
%   e1 <- parent.frame()
%   ## Evaluate a in e1
%   aa <- eval(expression(a), e1)
%   ## evaluate the expression bound to a in e1
%   a <- expression(x+y)
%   list(aa = aa, eval = eval(a, e1))
%}
%tst.ev <- function(a = 7) { x <- pi; y <- 1; ev() }
%tst.ev()  #-> aa : 7,  eval : 4.14
%
%a <- list(a = 3, b = 4)
%with(a, a <- 5) # alters the copy of a from the list, discarded.
%
%##
%## Example of evalq()
%##
%
%N <- 3
%env <- new.env()
%assign("N", 27, envir = env)
%## this version changes the visible copy of N only, since the argument
%## passed to eval is '4'.
%eval(N <- 4, env)
%N
%get("N", envir = env)
%## this version does the assignment in env, and changes N only there.
%evalq(N <- 5, env)
%N
%get("N", envir = env)
%
%
%##
%## Uses of local()
%##
%
%# Mutually recursive.
%# gg gets value of last assignment, an anonymous version of f.
%
%gg <- local({
%    k <- function(y)f(y)
%    f <- function(x) if(x) x*k(x-1) else 1
%})
%gg(10)
%sapply(1:5, gg)
%
%# Nesting locals: a is private storage accessible to k
%gg <- local({
%    k <- local({
%        a <- 1
%        function(y){print(a <<- a+1);f(y)}
%    })
%    f <- function(x) if(x) x*k(x-1) else 1
%})
%sapply(1:5, gg)
%
%ls(envir = environment(gg))
%ls(envir = environment(get("k", envir = environment(gg))))
%}
%\keyword{data}
%\keyword{programming}
%
\encoding{UTF8}
