% File src/library/base/man/funprog.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2008 R Core Team
% Distributed under GPL 2 or later
\name{funprog}
\alias{Filter}
\alias{Find}
\alias{Map}
\alias{Negate}
\alias{Reduce}
\alias{Position}
\title{Common Higher-Order Functions in Functional Programming Languages}
\description{
  \code{Reduce} は二項関数を使い，可能性として初期値を与えて与えられたベクトル
  の要素を次々に組み合わせる．
  \code{Filter} は真偽(論理値)関数が真であるようなベクトルの要素を抜き出す．
  \code{Find} と \code{Position} はベクトル中のそうした要素のそれぞれ最初か最後の要素と
  その位置を与える．\code{Map} は与えられたベクトルの対応する要素に関数を適用する．
  \code{Negate} は与えられた関数の否定を作り出す．
}
\usage{
Reduce(f, x, init, right = FALSE, accumulate = FALSE)
Filter(f, x)
Find(f, x, right = FALSE, nomatch = NULL)
Map(f, ...)
Negate(f)
Position(f, x, right = FALSE, nomatch = NA_integer_)
}
\arguments{
  \item{f}{複数の引数を持つ関数
    (\code{Reduce} には2項，\code{Filter}, \code{Find} そして
    \code{Position} には単項，もし \code{k} 個の引数で呼び出されたならば
    \code{Map} には \eqn{k} 項)．\code{Negate} には任意の真偽関数．}
  \item{x}{ベクトル．}
  \item{init}{\code{x} の要素と同じ種類の \R オブジェクト．}
  \item{right}{左から右へ(既定)か右から左ｈｗ進むかを指示する論理値．}
  \item{accumulate}{引き続く縮小組み合わせを蓄積するかどうかを指示する論理値．
    既定では最後の組み合わせだけが使われる．}
  \item{nomatch}{マッチするもの(真偽条件を満たす)が無いときに返される値．}
  \item{\dots}{ベクトル．}
}
\details{
  もし \code{init} が与えられると \code{Reduce} は 論理的にそれを
  それぞれ \code{x} の最初(左から右へ進む時)又は最後に付け加える．
  もしこの可能性として増補されたベクトル \eqn{v} が \eqn{n > 1} 個の要素を持てば，
  \code{Reduce} は \eqn{f} を \eqn{v} の要素にそれぞれ
  左から右又は右から左へ次々と適用する．つまり左縮約は
  \eqn{l_1 = f(v_1, v_2)}, \eqn{l_2 = f(l_1, v_3)} 等を計算し
  \eqn{l_{n-1} = f(l_{n-2}, v_n)} を返す．右縮約は
  \eqn{r_{n-1} = f(v_{n-1}, v_n)}, \eqn{r_{n-2} = f(v_{n-2}, r_{n-1})}
  とし \eqn{r_1 = f(v_1, r_2)} を返す．(つまり，もし \eqn{v} が
  系列 (2, 3, 4) で \eqn{f} が除算なら，左と右の縮約は
  \eqn{(2 / 3) / 4 = 1/6} と \eqn{2 / (3 / 4) = 8/3} をそれぞれ返す．)
  もし \eqn{v} がただ一つ要素を持てばそれが返される；
  もし要素が皆無なら \code{NULL} が返される．このように，
  \code{f} は常に二つの引数で呼び出されることが保証される．

  現在の実装は安定性とスケーラビリティを保証するため非再帰的である．

  \code{Reduce} は Common Lisp の \code{reduce} を真似ている．
  縮約はまた折込み(例えば Haskell で) や蓄積
  (例えば C++ Standard Template Library 中)としても知られている．
  蓄積バージョンは Haskell の scan 関数に対応する．

  \code{Filter} は単項真偽関数 \code{f} を \code{x} の各要素に適用する．
  必要なら論理値に変換し，これが真を返す \code{x} の部分集合を返す．
  存在するかもしれない \code{NA} 値は現在常に偽とされることを注意しよう；
  \code{NA} に対する制御は将来付け加えられるかもしれない．
  \code{Filter} は Haskell の \code{filter} や Common Lisp の
  \code{remove-if-not} に対応する．

  \code{Find} と \code{Position} はそれぞれ Common Lisp の
  \code{find-if} と \code{position-if} に対応する．もし真偽関数が真となる
  要素があれば，そうした要素の最初か最後の要素かその位置が，
  \code{right} が偽(既定)か真に応じてそれぞれ返される．もしそうした
  要素が皆無なら \code{nomatch} で指示された値が返される．現在の実装は
  パフォーマンスに対して最適化されていない．

  \code{Map} は \code{\link{mapply}} の単純なラッパであり結果の単純化を
  試みない．Common Lisp の \code{mapcar} に似る
  (しかしながら引数がリサイクルされる)．将来のバージョンは結果の型の制御
  を許すかもしれない．

  \code{Negate} は Common Lisp の \code{complement} に相当する．
  (真偽)関数 \code{f} をあたえた時，これは \code{f} が返すものの論理的否定
  を返す．
}

\seealso{
  パッケージ \pkg{parallel} 中の関数 \code{\link{clusterMap}} と
  \code{\link{mcmapply}} (Windows では無い)は \code{Map} の並列版を提供する．
}

\examples{
## 一般化加算器：
add <- function(x) Reduce("+", x)
add(list(1, 2, 3))
## sum() に似るが各縮小ステップで適当な '+' メソッドを使うので行列等にも使える．
## より一般的に，任意個数の引数に作用する多くの総称的関数が定義できる：
FOO <- function(...) Reduce(FOO2, list(...))
FOO2 <- function(x, y) UseMethod("FOO2")
## FOO() メソッドはそうすると FOO2() メソッド経由で提供できる．

## 一般的な累積加算器：
cadd <- function(x) Reduce("+", x, accumulate = TRUE)
cadd(seq_len(7))

## 連分数を計算する簡単な関数：
cfrac <- function(x) Reduce(function(u, v) u + 1 / v, x, right = TRUE)
## pi の連分数近似：
cfrac(c(3, 7, 15, 1, 292))
## オイラー数 (e) の連分数近似：
cfrac(c(2, 1, 2, 1, 1, 4, 1, 1, 6, 1, 1, 8))

## 関数の繰り返し適用：
Funcall <- function(f, ...) f(...)
## log(exp(acos(cos(0)) を計算する
Reduce(Funcall, list(log, exp, acos, cos), 0, right = TRUE)
## 関数の n 重繰り返し，汎関数スタイル：
Iterate <- function(f, n = 1)
    function(x) Reduce(Funcall, rep.int(list(f), n), x, right = TRUE)
## 黄金比の連分数禁じ：
Iterate(function(x) 1 + 1 / x, 30)(1)
## これは次と同じ
cfrac(rep.int(1, 31))
## x の平方根を初期値を与えた関数 t |-> (t + x / t) / 2 の不動点として計算：
asqrt <- function(x, n) Iterate(function(t) (t + x / t) / 2, n)
asqrt(2, 30)(10) # 正値ら出発 => +sqrt(2)
asqrt(2, 30)(-1) # 負値から出発 => -sqrt(2)

## 基本環境中のすべての関数を列挙：
funs <- Filter(is.function, sapply(ls(baseenv()), get, baseenv()))
## 10 以上の引数を持つ基本環境中の関数：
names(Filter(function(f) length(formals(args(f))) > 10, funs))
## 基本環境中の引数 '...' を持つ関数の数：
length(Filter(function(f)
              any(names(formals(args(f))) \%in\% "..."),
              funs))
\donttest{
## 基本環境中の関数でないすべてのオブジェクトを列挙：
Filter(Negate(is.function),  sapply(ls(baseenv()), get, baseenv()))
}}
\keyword{programming}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/funprog.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2008 R Core Team
%% Distributed under GPL 2 or later
%\name{funprog}
%\alias{Filter}
%\alias{Find}
%\alias{Map}
%\alias{Negate}
%\alias{Reduce}
%\alias{Position}
%\title{Common Higher-Order Functions in Functional Programming Languages}
%\description{
%  \code{Reduce} uses a binary function to successively combine the
%  elements of a given vector and a possibly given initial value.
%  \code{Filter} extracts the elements of a vector for which a predicate
%  (logical) function gives true.  \code{Find} and \code{Position} give
%  the first or last such element and its position in the vector,
%  respectively.  \code{Map} applies a function to the corresponding
%  elements of given vectors.  \code{Negate} creates the negation of a
%  given function.
%}
%\usage{
%Reduce(f, x, init, right = FALSE, accumulate = FALSE)
%Filter(f, x)
%Find(f, x, right = FALSE, nomatch = NULL)
%Map(f, ...)
%Negate(f)
%Position(f, x, right = FALSE, nomatch = NA_integer_)
%}
%\arguments{
%  \item{f}{a function of the appropriate arity (binary for
%    \code{Reduce}, unary for \code{Filter}, \code{Find} and
%    \code{Position}, \eqn{k}-ary for \code{Map} if this is called with
%    \eqn{k} arguments).  An arbitrary predicate function for
%    \code{Negate}.}
%  \item{x}{a vector.}
%  \item{init}{an \R object of the same kind as the elements of
%    \code{x}.}
%  \item{right}{a logical indicating whether to proceed from left to
%    right (default) or from right to left.}
%  \item{accumulate}{a logical indicating whether the successive reduce
%    combinations should be accumulated.  By default, only the final
%    combination is used.}
%  \item{nomatch}{the value to be returned in the case when
%    \dQuote{no match} (no element satisfying the predicate) is found.}
%  \item{\dots}{vectors.}
%}
%\details{
%  If \code{init} is given, \code{Reduce} logically adds it to the start
%  (when proceeding left to right) or the end of \code{x}, respectively.
%  If this possibly augmented vector \eqn{v} has \eqn{n > 1} elements,
%  \code{Reduce} successively applies \eqn{f} to the elements of \eqn{v}
%  from left to right or right to left, respectively.  I.e., a left
%  reduce computes \eqn{l_1 = f(v_1, v_2)}, \eqn{l_2 = f(l_1, v_3)}, etc.,
%  and returns \eqn{l_{n-1} = f(l_{n-2}, v_n)}, and a right reduce does
%  \eqn{r_{n-1} = f(v_{n-1}, v_n)}, \eqn{r_{n-2} = f(v_{n-2}, r_{n-1})}
%  and returns \eqn{r_1 = f(v_1, r_2)}.  (E.g., if \eqn{v} is the
%  sequence (2, 3, 4) and \eqn{f} is division, left and right reduce give
%  \eqn{(2 / 3) / 4 = 1/6} and \eqn{2 / (3 / 4) = 8/3}, respectively.)
%  If \eqn{v} has only a single element, this is returned; if there are
%  no elements, \code{NULL} is returned.  Thus, it is ensured that
%  \code{f} is always called with 2 arguments.
%
%  The current implementation is non-recursive to ensure stability and
%  scalability.
%
%  \code{Reduce} is patterned after Common Lisp's \code{reduce}.  A
%  reduce is also known as a fold (e.g., in Haskell) or an accumulate
%  (e.g., in the C++ Standard Template Library).  The accumulative
%  version corresponds to Haskell's scan functions.
%
%  \code{Filter} applies the unary predicate function \code{f} to each
%  element of \code{x}, coercing to logical if necessary, and returns the
%  subset of \code{x} for which this gives true.  Note that possible
%  \code{NA} values are currently always taken as false; control over
%  \code{NA} handling may be added in the future.  \code{Filter}
%  corresponds to \code{filter} in Haskell or \code{remove-if-not} in
%  Common Lisp.
%
%  \code{Find} and \code{Position} are patterned after Common Lisp's
%  \code{find-if} and \code{position-if}, respectively.  If there is an
%  element for which the predicate function gives true, then the first or
%  last such element or its position is returned depending on whether
%  \code{right} is false (default) or true, respectively.  If there is no
%  such element, the value specified by \code{nomatch} is returned.  The
%  current implementation is not optimized for performance.
%
%  \code{Map} is a simple wrapper to \code{\link{mapply}} which does not
%  attempt to simplify the result, similar to Common Lisp's \code{mapcar}
%  (with arguments being recycled, however).  Future versions may allow
%  some control of the result type.
%
%  \code{Negate} corresponds to Common Lisp's \code{complement}.  Given a
%  (predicate) function \code{f}, it creates a function which returns the
%  logical negation of what \code{f} returns.
%}
%
%\seealso{
%  Function \code{\link{clusterMap}} and \code{\link{mcmapply}} (not
%  Windows) in package \pkg{parallel} provide parallel versions of \code{Map}.
%}
%
%\examples{
%## A general-purpose adder:
%add <- function(x) Reduce("+", x)
%add(list(1, 2, 3))
%## Like sum(), but can also used for adding matrices etc., as it will
%## use the appropriate '+' method in each reduction step.
%## More generally, many generics meant to work on arbitrarily many
%## arguments can be defined via reduction:
%FOO <- function(...) Reduce(FOO2, list(...))
%FOO2 <- function(x, y) UseMethod("FOO2")
%## FOO() methods can then be provided via FOO2() methods.
%
%## A general-purpose cumulative adder:
%cadd <- function(x) Reduce("+", x, accumulate = TRUE)
%cadd(seq_len(7))
%
%## A simple function to compute continued fractions:
%cfrac <- function(x) Reduce(function(u, v) u + 1 / v, x, right = TRUE)
%## Continued fraction approximation for pi:
%cfrac(c(3, 7, 15, 1, 292))
%## Continued fraction approximation for Euler's number (e):
%cfrac(c(2, 1, 2, 1, 1, 4, 1, 1, 6, 1, 1, 8))
%
%## Iterative function application:
%Funcall <- function(f, ...) f(...)
%## Compute log(exp(acos(cos(0))
%Reduce(Funcall, list(log, exp, acos, cos), 0, right = TRUE)
%## n-fold iterate of a function, functional style:
%Iterate <- function(f, n = 1)
%    function(x) Reduce(Funcall, rep.int(list(f), n), x, right = TRUE)
%## Continued fraction approximation to the golden ratio:
%Iterate(function(x) 1 + 1 / x, 30)(1)
%## which is the same as
%cfrac(rep.int(1, 31))
%## Computing square root approximations for x as fixed points of the
%## function t |-> (t + x / t) / 2, as a function of the initial value:
%asqrt <- function(x, n) Iterate(function(t) (t + x / t) / 2, n)
%asqrt(2, 30)(10) # Starting from a positive value => +sqrt(2)
%asqrt(2, 30)(-1) # Starting from a negative value => -sqrt(2)
%
%## A list of all functions in the base environment:
%funs <- Filter(is.function, sapply(ls(baseenv()), get, baseenv()))
%## Functions in base with more than 10 arguments:
%names(Filter(function(f) length(formals(args(f))) > 10, funs))
%## Number of functions in base with a '...' argument:
%length(Filter(function(f)
%              any(names(formals(args(f))) \%in\% "..."),
%              funs))
%\donttest{
%## Find all objects in the base environment which are *not* functions:
%Filter(Negate(is.function),  sapply(ls(baseenv()), get, baseenv()))
%}}
%\keyword{programming}
%
\encoding{UTF8}
