% File src/library/stats/man/fft.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{fft}
\alias{fft}
\alias{mvfft}
%%\title{Fast Discrete Fourier Transform (FFT)}
\title{離散高速フーリエ変換}
\description{
  配列の離散高速フーリエ変換(DFT) を高速なアルゴリズム
  \dQuote{高速フーリエ変換} (FFT)で計算する．
}
\usage{
fft(z, inverse = FALSE)
mvfft(z, inverse = FALSE)
}
\arguments{
  \item{z}{変換される値を含む実又は複素数値配列．}
  \item{inverse}{もし \code{TRUE} ならば正規化されない
    逆変換が計算される
    (逆変換は \eqn{e} の指数に \code{+} を持つが
    \code{1/length(x)}) で割らない．}
}
\value{
  \code{z} がベクトルならば，
  \code{fft} が計算し返す値は \code{z} 中の値の数列の
  正規化されない一変量離散フーリエ変換である．
  特に \code{y <- fft(z)} は \eqn{h = 1,\ldots,n}{h = 1, ..., n}
  に対して
  \deqn{y[h] = \sum_{k=1}^n z[k] \exp(-2\pi i (k-1) (h-1)/n)}{%
        y[h] =  sum_{k=1}^n z[k]*exp(-2*pi*1i*(k-1)*(h-1)/n)}
  を返す，ここで n = \code{length(y)} である．もし
  \code{inverse} が \code{TRUE} ならば \eqn{\exp(-2\pi\ldots)}{exp(-2*pi...)}
  は \eqn{\exp(2\pi\ldots)}{exp(2*pi...)} で置き換えられる．

  \code{z} が配列を含む場合，\code{fft} は多変量(空間)変換を
  計算し返す．
  もし \code{inverse} が \code{TRUE} ならば
  (正規化されない)逆フーリエ変換が返される，つまり
  もし \code{y <- fft(z)} ならば \code{z} は
  \code{fft(y, inverse = TRUE) / length(y)} である．

  対照的に \code{mvfft} は実もしくは複素数値行列を引数に取り
  同形の行列を返すが，各列はその離散フーリエ変換で置き換えられる．
  これはベクトル値系列の解析に有用である．

  FFT は変換される長さが高度に合成的
  (つまり多くの約数を持つ)とき最速である．
  もしそうでなければ変換は計算に時間がかかり
  大量のメモリを使用する．
}
\source{
  Singleton (1979) 中の Fortran コードの C 翻訳を使う．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.

  Singleton, R. C. (1979)
  Mixed Radix Fast Fourier Transforms,
  in \emph{Programs for Digital Signal Processing},
  IEEE Digital Signal Processing Committee eds.
  IEEE Press.

  Cooley, James W., and Tukey, John W. (1965)
  An algorithm for the machine calculation of complex Fourier series,
  \emph{Math. Comput.} \bold{19}(90), 297--301.
  \url{https://dx.doi.org/10.1090/S0025-5718-1965-0178586-1}
}
\seealso{
  \code{\link{convolve}}, \code{\link{nextn}}.
}
\examples{
x <- 1:4
fft(x)
fft(fft(x), inverse = TRUE)/length(x)

## 遅い離散フーリエ変換(DFT) - 例えば公式のチェック用
fft0 <- function(z, inverse=FALSE) {
  n <- length(z)
  if(n == 0) return(z)
  k <- 0:(n-1)
  ff <- (if(inverse) 1 else -1) * 2*pi * 1i * k/n
  vapply(1:n, function(h) sum(z * exp(ff*(h-1))), complex(1))
}

relD <- function(x,y) 2* abs(x - y) / abs(x + y)
n <- 2^8
z <- complex(n, rnorm(n), rnorm(n))
\donttest{## 相対誤差は 4*10^{-14} のオーダー：
summary(relD(fft(z), fft0(z)))
summary(relD(fft(z, inverse=TRUE), fft0(z, inverse=TRUE)))
}}
\keyword{math}
\keyword{dplot}

\encoding{UTF-8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/stats/man/fft.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{fft}
%\alias{fft}
%\alias{mvfft}
%\title{Fast Discrete Fourier Transform (FFT)}
%\description{
%  Computes the Discrete Fourier Transform (DFT) of an array with a fast
%  algorithm, the \dQuote{Fast Fourier Transform} (FFT).
%}
%\usage{
%fft(z, inverse = FALSE)
%mvfft(z, inverse = FALSE)
%}
%\arguments{
%  \item{z}{a real or complex array containing the values to be
%    transformed.}
%  \item{inverse}{if \code{TRUE}, the unnormalized inverse transform is
%    computed (the inverse has a \code{+} in the exponent of \eqn{e},
%    but here, we do \emph{not} divide by \code{1/length(x)}).}
%}
%\value{
%  When \code{z} is a vector, the value computed and returned by
%  \code{fft} is the unnormalized univariate discrete Fourier transform of the
%  sequence of values in \code{z}.  Specifically, \code{y <- fft(z)} returns
%  \deqn{y[h] = \sum_{k=1}^n z[k] \exp(-2\pi i (k-1) (h-1)/n)}{%
%        y[h] =  sum_{k=1}^n z[k]*exp(-2*pi*1i*(k-1)*(h-1)/n)}
%  for \eqn{h = 1,\ldots,n}{h = 1, ..., n} where n = \code{length(y)}.  If
%  \code{inverse} is \code{TRUE}, \eqn{\exp(-2\pi\ldots)}{exp(-2*pi...)}
%  is replaced with \eqn{\exp(2\pi\ldots)}{exp(2*pi...)}.
%
%  When \code{z} contains an array, \code{fft} computes and returns the
%  multivariate (spatial) transform.  If \code{inverse} is \code{TRUE},
%  the (unnormalized) inverse Fourier transform is returned, i.e.,
%  if \code{y <- fft(z)}, then \code{z} is
%  \code{fft(y, inverse = TRUE) / length(y)}.
%
%  By contrast, \code{mvfft} takes a real or complex matrix as argument,
%  and returns a similar shaped matrix, but with each column replaced by
%  its discrete Fourier transform.  This is useful for analyzing
%  vector-valued series.
%
%  The FFT is fastest when the length of the series being transformed
%  is highly composite (i.e., has many factors).  If this is not the
%  case, the transform may take a long time to compute and will use a
%  large amount of memory.
%}
%\source{
%  Uses C translation of Fortran code in Singleton (1979).
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%
%  Singleton, R. C. (1979)
%  Mixed Radix Fast Fourier Transforms,
%  in \emph{Programs for Digital Signal Processing},
%  IEEE Digital Signal Processing Committee eds.
%  IEEE Press.
%
%  Cooley, James W., and Tukey, John W. (1965)
%  An algorithm for the machine calculation of complex Fourier series,
%  \emph{Math. Comput.} \bold{19}(90), 297--301.
%  \url{https://dx.doi.org/10.1090/S0025-5718-1965-0178586-1}
%}
%\seealso{
%  \code{\link{convolve}}, \code{\link{nextn}}.
%}
%\examples{
%x <- 1:4
%fft(x)
%fft(fft(x), inverse = TRUE)/length(x)
%
%## Slow Discrete Fourier Transform (DFT) - e.g., for checking the formula
%fft0 <- function(z, inverse=FALSE) {
%  n <- length(z)
%  if(n == 0) return(z)
%  k <- 0:(n-1)
%  ff <- (if(inverse) 1 else -1) * 2*pi * 1i * k/n
%  vapply(1:n, function(h) sum(z * exp(ff*(h-1))), complex(1))
%}
%
%relD <- function(x,y) 2* abs(x - y) / abs(x + y)
%n <- 2^8
%z <- complex(n, rnorm(n), rnorm(n))
%\donttest{## relative differences in the order of 4*10^{-14} :
%summary(relD(fft(z), fft0(z)))
%summary(relD(fft(z, inverse=TRUE), fft0(z, inverse=TRUE)))
%}}
%\keyword{math}
%\keyword{dplot}
%
%\encoding{UTF-8}
%
